// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';

part 'text_input_state.dart';

class TextInputCubit extends Cubit<TextInputState> {
  TextInputCubit() : super(const TextInputState.initial());

  // ControlState logic
  FutureOr<void> onFocus() async {
    if (state.controlState != ControlState.disabled) {
      emit(state.copyWith(controlState: ControlState.focused));
    }
  }

  FutureOr<void> onUnfocus() async {
    if (state.controlState != ControlState.disabled) {
      emit(state.copyWith(controlState: ControlState.normal));
    }
  }

  Future<void> disable() async {
    emit(state.copyWith(controlState: ControlState.disabled));
  }

  Future<void> enable() async {
    emit(state.copyWith(controlState: ControlState.normal));
  }

  // StatusState logic
  FutureOr<void> onInvalid(String? error) async {
    emit(state.copyWith(statusState: StatusState.error, statusMessage: error));
  }

  FutureOr<void> onSuccess() async {
    emit(state.copyWith(statusState: StatusState.initial, statusMessage: ''));
  }
}
